-- ParticleShop
-- v0.1 by Lixquid

local ww, wh = love.graphics.getWidth(), love.graphics.getHeight()

-- Helper Functions
local function inRect( mx, my, x, y, w, h )
	return mx >= x and mx <= x + w and my >= y and my <= y + h
end

local function oprint( t, x, y, c, oc )
	love.graphics.setColor( oc )
	for i = -1, 1 do
		for j = -1, 1 do
			love.graphics.print( t, x + i, y + j )
		end
	end
	love.graphics.setColor( c )
	love.graphics.print( t, x, y )
end

local function clamp( n, mn, mx )
	return math.floor( math.min( mx, math.max( n, mn ) ) * 100 + 0.5 ) / 100
end

-- Tooltips
tooltips = {
	[ "" ] = "",
	main_sysstart = "Start the particle system",
	main_sysstop = "Stop the particle system",
	main_sysreset = "Reset the particle system",
	main_play = "Resume particle system calculation",
	main_pause = "Pause particle system calculation",
	main_step = "Advance particle system calculation one frame",
	main_speed = "Particle system calculation speed settings",
	main_particle = "Particle settings",
	
	speed_speed = "Speed of particle system calculation",
	
	particle_gravity = "Acceleration of particle in y-axis [min,max]",
	particle_life = "Lifetime of the particle [min,max]"
}

function love.load()
	
	-- Load External Resources
	img_glow = love.graphics.newImage( "img/glow.png" )
	img_play = love.graphics.newImage( "img/play.png" )
	img_pause = love.graphics.newImage( "img/pause.png" )
	img_step = love.graphics.newImage( "img/step.png" )
	img_speed = love.graphics.newImage( "img/speed.png" )
	img_sysstart = love.graphics.newImage( "img/sysstart.png" )
	img_sysstop = love.graphics.newImage( "img/sysstop.png" )
	img_sysreset = love.graphics.newImage( "img/sysreset.png" )
	img_particle = love.graphics.newImage( "img/particle.png" )
	img_system = love.graphics.newImage( "img/system.png" )
	img_colors = love.graphics.newImage( "img/colors.png" )
	
	font_gridblock = love.graphics.newFont( "gridblock.ttf", 8 )
	
	-- Default Settings
	s = {}
	s.play = true
	s.step = false
	s.speed = 1
	
	s.blendMode = "additive"
	
	s.gravitymin = 0
	s.gravitymax = 0
	s.lifemin = 1
	s.lifemax = 1
	
	-- Internal Variables
	closeok = false
	closetimer = 0
	closesaved = false
	fps = 100
	cwin = ""
	activee = ""
	
	anim_auxback = 0
	
	-- Particle Sprites
	pimgs = {}
	pimgs[ "part1" ] = love.graphics.newImage( "particle/part1.png" )
	
	-- Set up the ParticleSystem
	p = love.graphics.newParticleSystem( pimgs[ "part1" ], 100 )
	p:setEmissionRate( 100 )
	p:setSpeed(300, 400)
	p:setGravity(0)
	p:setSizes(2, 1)
	p:setColors(255, 255, 255, 255, 58, 128, 255, 0)
	p:setPosition(400, 300)
	p:setParticleLife(1)
	p:setDirection(0)
	p:setSpread(360)
	p:setRadialAcceleration(-2000)
	p:setTangentialAcceleration(1000)
	p:start()
	
end

function love.draw()
	
	love.graphics.setFont( font_gridblock )
	love.graphics.setLineStyle( "rough" )
	love.graphics.setLineWidth( 1 )
	
	-- Draw the ParticleSystem
	love.graphics.setBlendMode( s.blendMode )
	love.graphics.draw( p, 0, 0 )
	
	-- Draw the GUI
	
	-- Aux Backplate
	
	if anim_auxback > 0.01 then
		
		love.graphics.setBlendMode( "subtractive" )
		love.graphics.setColor( 255, 255, 255, 180 * anim_auxback )
		love.graphics.rectangle( "fill", 0, 26, love.graphics.getWidth(), 26 )
		
		love.graphics.setBlendMode( "alpha" )
		love.graphics.setColor( 255, 255, 255, 40 * anim_auxback )
		love.graphics.rectangle( "fill", 0, 26, love.graphics.getWidth(), 26 )
		
		love.graphics.setBlendMode( "alpha" )
		love.graphics.setColor( 0, 0, 0, 100 * anim_auxback )
		love.graphics.rectangle( "fill", 0, 51, love.graphics.getWidth(), 1 )
		
		for i = 0, 23 do
			love.graphics.setColor( 0, 0, 0, i * 4 * anim_auxback )
			love.graphics.rectangle( "fill", 0, 26 + i, love.graphics.getWidth(), 1 )
		end
		
	end
	
	-- Speed Control GUI
	
	if cwin == "speed" then
		
		love.graphics.setColor( 255, 255, 255, 100 * anim_auxback * ( activee == "speed_speed" and 1.5 or 1 ) )
		love.graphics.rectangle( "line", 5, 30, 100, 16 )
		love.graphics.rectangle( "fill", 6, 31, math.min( 48.5 * s.speed, 97 ), 13 )
		oprint( s.speed, 8, 34, { 255, 255, 255, activee == "speed_speed" and 255 or 0 }, { 0, 0, 0, activee == "speed_speed" and 255 or 0 } )
		
	end
	
	-- Particle Settings GUI
	
	if cwin == "particle" then
		
		-- Gravity
		love.graphics.setColor( 255, 255, 255, 100 * anim_auxback * ( activee == "particle_gravity" and 1.5 or 1 ) )
		love.graphics.rectangle( "line", 5, 30, 100, 16 )
		love.graphics.rectangle( "fill", 6, 31, math.min( 50 + s.gravitymin / 200, 97 ), 6 )
		love.graphics.rectangle( "fill", 6, 38, math.min( 50 + s.gravitymax / 200, 97 ), 6 )
		oprint( string.format( "%s - %s", s.gravitymin, s.gravitymax ), 8, 34, { 255, 255, 255, activee == "particle_gravity" and 255 or 0 }, { 0, 0, 0, activee == "particle_gravity" and 255 or 0 } )
		
		-- Life
		love.graphics.setColor( 255, 255, 255, 100 * anim_auxback * ( activee == "particle_life" and 1.5 or 1 ) )
		love.graphics.rectangle( "line", 110, 30, 100, 16 )
		love.graphics.rectangle( "fill", 111, 31, math.min( s.lifemin * 9.7, 97 ), 6 )
		love.graphics.rectangle( "fill", 111, 38, math.min( s.lifemax * 9.7, 97 ), 6 )
		oprint( string.format( "%s - %s", s.lifemin, s.lifemax ), 113, 34, { 255, 255, 255, activee == "particle_life" and 255 or 0 }, { 0, 0, 0, activee == "particle_life" and 255 or 0 } )
		
	end
	
	-- Backplate
	
	love.graphics.setBlendMode( "subtractive" )
	love.graphics.setColor( 255, 255, 255, 180 )
	love.graphics.rectangle( "fill", 0, 0, love.graphics.getWidth(), 26 )
	
	love.graphics.setBlendMode( "alpha" )
	love.graphics.setColor( 255, 255, 255, 40 )
	love.graphics.rectangle( "fill", 0, 0, love.graphics.getWidth(), 26 )
	
	love.graphics.setBlendMode( "alpha" )
	love.graphics.setColor( 0, 0, 0, 100 )
	love.graphics.rectangle( "fill", 0, 25, love.graphics.getWidth(), 1 )
	
	for i = 0, 23 do
		love.graphics.setColor( 0, 0, 0, i * 4 )
		love.graphics.rectangle( "fill", 0, i, love.graphics.getWidth(), 1 )
	end
	
	-- Playback
	love.graphics.setColor( 255, 255, 255, 255 )
	
	love.graphics.draw( img_sysstart, 5, 5 )
	if p:isActive() then
		love.graphics.setBlendMode( "additive" )
		love.graphics.setColor( 80, 80, 80, 255 )
		love.graphics.draw( img_glow, 5, 5 )
		love.graphics.setBlendMode( "alpha" )
		love.graphics.setColor( 255, 255, 255, 255 )
	end
	love.graphics.draw( img_sysstop, 26, 5 )
	if not p:isActive() then
		love.graphics.setBlendMode( "additive" )
		love.graphics.setColor( 80, 80, 80, 255 )
		love.graphics.draw( img_glow, 26, 5 )
		love.graphics.setBlendMode( "alpha" )
		love.graphics.setColor( 255, 255, 255, 255 )
	end
	love.graphics.draw( img_sysreset, 47, 5 )
	
	love.graphics.setColor( 255, 255, 255, 30 )
	love.graphics.rectangle( "fill", 68, 5, 1, 16 )
	love.graphics.setColor( 0, 0, 0, 100 )
	love.graphics.rectangle( "fill", 69, 5, 1, 16 )
	
	love.graphics.setColor( 255, 255, 255, 255 )
	
	love.graphics.draw( img_play, 74, 5 )
	if s.play then
		love.graphics.setBlendMode( "additive" )
		love.graphics.setColor( 80, 80, 80, 255 )
		love.graphics.draw( img_glow, 74, 5 )
		love.graphics.setBlendMode( "alpha" )
		love.graphics.setColor( 255, 255, 255, 255 )
	end
	love.graphics.draw( img_pause, 95, 5 )
	if not s.play then
		love.graphics.setBlendMode( "additive" )
		love.graphics.setColor( 80, 80, 80, 255 )
		love.graphics.draw( img_glow, 95, 5 )
		love.graphics.setBlendMode( "alpha" )
		love.graphics.setColor( 255, 255, 255, 255 )
	end
	love.graphics.draw( img_step, 116, 5 )
	love.graphics.draw( img_speed, 137, 5 )
	if cwin == "speed" then
		love.graphics.setBlendMode( "additive" )
		love.graphics.setColor( 80, 80, 80, 255 )
		love.graphics.draw( img_glow, 137, 5 )
		love.graphics.setBlendMode( "alpha" )
		love.graphics.setColor( 255, 255, 255, 255 )
	end
	
	love.graphics.setColor( 255, 255, 255, 30 )
	love.graphics.rectangle( "fill", 158, 5, 1, 16 )
	love.graphics.setColor( 0, 0, 0, 100 )
	love.graphics.rectangle( "fill", 159, 5, 1, 16 )
	
	love.graphics.setColor( 255, 255, 255, 255 )
	love.graphics.draw( img_particle, 164, 5 )
	if cwin == "particle" then
		love.graphics.setBlendMode( "additive" )
		love.graphics.setColor( 80, 80, 80, 255 )
		love.graphics.draw( img_glow, 164, 5 )
		love.graphics.setBlendMode( "alpha" )
		love.graphics.setColor( 255, 255, 255, 255 )
	end
	
	-- Tooltip
	oprint( tooltips[ activee ], 5, love.graphics.getHeight() - 11, { 255, 255, 255, 255 }, { 0, 0, 0, 255 } )
	
	if closeok then
		
		love.graphics.setBlendMode( "subtractive" )
		love.graphics.setColor( 100, 255, 255, math.min( ( love.timer.getMicroTime() - closetimer ) * 150, 200 ) )
		love.graphics.rectangle( "fill", 0, 0, love.graphics.getWidth(), love.graphics.getHeight() )
		
		love.graphics.setBlendMode( "alpha" )
		love.graphics.setColor( 80, 80, 80, math.min( ( love.timer.getMicroTime() - closetimer ) * 150, 200 ) )
		love.graphics.rectangle( "fill", love.graphics.getWidth() / 2 - 150, love.graphics.getHeight() / 2 - 30, 300, 60 )
		love.graphics.setColor( 255, 255, 255, math.min( ( love.timer.getMicroTime() - closetimer ) * 75, 100 ) )
		love.graphics.rectangle( "line", love.graphics.getWidth() / 2 - 150, love.graphics.getHeight() / 2 - 30, 300, 60 )
		
		love.graphics.setColor( 255, 255, 255, math.min( ( love.timer.getMicroTime() - closetimer ) * 150, 200 ) )
		love.graphics.print( "You have unsaved changes.\nAre you sure you want to quit?", love.graphics.getWidth() / 2 - 145, love.graphics.getHeight() / 2 - 25 )
		
		love.graphics.rectangle( "line", love.graphics.getWidth() / 2 - 60, love.graphics.getHeight() / 2 + 5, 100, 20 )
		love.graphics.rectangle( "line", love.graphics.getWidth() / 2 + 45, love.graphics.getHeight() / 2 + 5, 100, 20 )
		
		love.graphics.print( "Quit", love.graphics.getWidth() / 2 - 55, love.graphics.getHeight() / 2 + 11 )
		love.graphics.print( "Cancel", love.graphics.getWidth() / 2 + 50, love.graphics.getHeight() / 2 + 11 )
		
		love.graphics.setColor( 255, 255, 255, math.min( ( love.timer.getMicroTime() - closetimer ) * 75, 50 ) )
		if inRect( love.mouse.getX(), love.mouse.getY(), love.graphics.getWidth() / 2 - 60, love.graphics.getHeight() / 2 + 5, 100, 20 ) then
			love.graphics.rectangle( "fill", love.graphics.getWidth() / 2 - 60, love.graphics.getHeight() / 2 + 5, 100, 20 )
		elseif inRect( love.mouse.getX(), love.mouse.getY(), love.graphics.getWidth() / 2 + 45, love.graphics.getHeight() / 2 + 5, 100, 20 ) then
			love.graphics.rectangle( "fill", love.graphics.getWidth() / 2 + 45, love.graphics.getHeight() / 2 + 5, 100, 20 )
		end
		
	end
	
end

function love.update( dt )
	
	local x, y = love.mouse.getX(), love.mouse.getY()
	fps = fps + ( 1 / dt - fps ) / 10
	love.graphics.setCaption( "ParticleShop v0.1 - FPS: " .. math.floor( fps + 0.5 ) )
	
	if closeok then return end
	
	-- Set ParticleSystem position
	if love.mouse.isDown( "l" ) and y > 26 + ( cwin ~= "" and 26 or 0 ) then
		p:setPosition( x, y )
	end
	
	-- Update system if not paused
	if s.play or s.step then
		p:update( dt * s.speed )
		s.step = false
	end
	
	-- Extreme Lag Safeguard
	if fps < 1 then
		s.play = false
	end
	
	-- ACTIVE ELEMENTS --
	
	-- Main Bar
	if inRect( x, y, 5, 5, 16, 16 ) then
		activee = "main_sysstart"
	elseif inRect( x, y, 26, 5, 16, 16 ) then
		activee = "main_sysstop"
	elseif inRect( x, y, 47, 5, 16, 16 ) then
		activee = "main_sysreset"
	elseif inRect( x, y, 74, 5, 16, 16 ) then
		activee = "main_play"
	elseif inRect( x, y, 95, 5, 16, 16 ) then
		activee = "main_pause"
	elseif inRect( x, y, 116, 5, 16, 16 ) then
		activee = "main_step"
	elseif inRect( x, y, 137, 5, 16, 16 ) then
		activee = "main_speed"
	elseif inRect( x, y, 164, 5, 16, 16 ) then
		activee = "main_particle"
	
	-- Speed Settings
	elseif cwin == "speed" and inRect( x, y, 5, 30, 100, 16 ) then
		activee = "speed_speed"
	
	-- Particle Settings
	elseif cwin == "particle" and inRect( x, y, 5, 30, 100, 16 ) then
		activee = "particle_gravity"
	elseif cwin == "particle" and inRect( x, y, 110, 30, 100, 16 ) then
		activee = "particle_life"
	
	-- Nothing much going on, huh?
	else
		activee = ""
	end
	
	-- DYNAMIC SETTING ADJUSTMENT --
	
	if love.mouse.isDown( "l" ) then
		
		if cwin == "speed" then
			
			if inRect( x, y, 5, 30, 100, 16 ) then
				
				s.speed = ( x - 5 ) / 50
				
			end
			
		end
		
		if cwin == "particle" then
			
			if inRect( x, y, 5, 30, 100, 16 ) then
				
				s.gravitymin = ( x - 55 ) * 200
				s.gravitymax = math.max( s.gravitymax, s.gravitymin )
				p:setGravity( s.gravitymin, s.gravitymax )
				
			end
			
			if inRect( x, y, 110, 30, 100, 16 ) then
				
				s.lifemin = ( x - 110 ) / 10
				s.lifemax = math.max( s.lifemax, s.lifemin )
				p:setParticleLife( s.lifemin, s.lifemax )
				
			end
			
		end
		
	end
	
	if love.mouse.isDown( "r" ) then
		
		if cwin == "particle" then
			
			if inRect( x, y, 5, 30, 100, 16 ) then
				
				s.gravitymax = ( x - 55 ) * 200
				s.gravitymin = math.min( s.gravitymax, s.gravitymin )
				p:setGravity( s.gravitymin, s.gravitymax )
				
			end
			
			if inRect( x, y, 110, 30, 100, 16 ) then
				
				s.lifemax = ( x - 110 ) / 10
				s.lifemin = math.min( s.lifemax, s.lifemin )
				p:setParticleLife( s.lifemin, s.lifemax )
				
			end
			
		end
		
	end
	
	-- SETTING CLAMPING --
	s.speed = clamp( s.speed, 0.02, 2 )
	s.gravitymin = clamp( s.gravitymin, -10000, 10000 )
	s.gravitymax = clamp( s.gravitymax, -10000, 10000 )
	s.lifemin = clamp( s.lifemin, 0, 10 )
	s.lifemax = clamp( s.lifemax, 0, 10 )
	
	-- Animation Calculation
	anim_auxback = anim_auxback + ( ( cwin ~= "" and 1 or 0 ) - anim_auxback ) / 10
	
end

function love.mousereleased( x, y, but )
	
	if closeok then
		if but == "l" then
			if inRect( x, y, love.graphics.getWidth() / 2 - 60, love.graphics.getHeight() / 2 + 5, 100, 20 ) then
				love.event.push( "quit" )
			elseif inRect( x, y, love.graphics.getWidth() / 2 + 45, love.graphics.getHeight() / 2 + 5, 100, 20 ) then
				closeok = false
			end
		end
		return
	end
	
	if but == "l" then
		
		-- Start
		if inRect( x, y, 5, 5, 16, 16 ) then
			p:start()
		end
		
		-- Stop
		if inRect( x, y, 26, 5, 16, 16 ) then
			p:stop()
		end
		
		-- Reset
		if inRect( x, y, 47, 5, 16, 16 ) then
			p:reset()
		end
		
		-- Play
		if inRect( x, y, 74, 5, 16, 16 ) then
			s.play = true
		end
		
		-- Pause
		if inRect( x, y, 95, 5, 16, 16 ) then
			s.play = false
		end
		
		-- Step
		if inRect( x, y, 116, 5, 16, 16 ) then
			s.step = true
			s.play = false
		end
		
		-- Speed settings
		if inRect( x, y, 137, 5, 16, 16 ) then
			cwin = cwin == "speed" and "" or "speed"
		end
		
		-- Particle settings
		if inRect( x, y, 164, 5, 16, 16 ) then
			cwin = cwin == "particle" and "" or "particle"
		end
		
	end
	
end

function love.keypressed( key )
	
	-- Reload
	if key == "r" then
		love.filesystem.load( "main.lua" )()
		love.load()
	end
	
	-- Finetune Controls
	if key == "right" or key == "left" or key == "up" or key == "down" then
		
		local fac = ( key == "right" or key == "up" ) and 1 or -1
		local mod = key == "up" or key == "down"
		
		if activee == "speed_speed" then
			s.speed = s.speed + 0.02 * fac
		end
		
		if activee == "particle_gravity" then
			if mod then
				s.gravitymax = s.gravitymax + 200 * fac
				s.gravitymin = math.min( s.gravitymax, s.gravitymin )
				p:setGravity( s.gravitymin, s.gravitymax )
			else
				s.gravitymin = s.gravitymin + 200 * fac
				s.gravitymax = math.max( s.gravitymax, s.gravitymin )
				p:setGravity( s.gravitymin, s.gravitymax )
			end
		end
		
		if activee == "particle_life" then
			if mod then
				s.lifemax = s.lifemax + 0.1 * fac
				s.lifemin = math.min( s.lifemax, s.lifemin )
				p:setParticleLife( s.lifemin, s.lifemax )
			else
				s.lifemin = s.lifemin + 0.1 * fac
				s.lifemax = math.max( s.lifemax, s.lifemin )
				p:setParticleLife( s.lifemin, s.lifemax )
			end
		end
		
	end
	
end

function love.quit()
	
	if not closeok and not closesaved then
		
		closeok = true
		closetimer = love.timer.getMicroTime()
		return true
		
	end
	
end