function love.load()
	
	-- Load GUI images
	img_play = love.graphics.newImage( "img/play.png" )
	img_play_on = love.graphics.newImage( "img/play_on.png" )
	img_pause = love.graphics.newImage( "img/pause.png" )
	img_pause_on = love.graphics.newImage( "img/pause_on.png" )
	img_step = love.graphics.newImage( "img/step.png" )
	
	font_gridblock = love.graphics.newFont( "gridblock.ttf", 8 )
	
	-- ParticleSystem settings (as well as calculation settings)
	s = {}
	s.play = true
	s.step = false
	
	-- Particle Images
	pimgs = {}
	pimgs[ "part1" ] = love.graphics.newImage( "particle/part1.png" )
	
	-- Setup the default ParticleSystem
	p = love.graphics.newParticleSystem( pimgs[ "part1" ], 1000 )
	p:setEmissionRate(100)
	p:setSpeed(300, 400)
	p:setGravity(0)
	p:setSizes(2, 1)
	p:setColors(255, 255, 255, 255, 58, 128, 255, 0)
	p:setPosition(150, 100)
	p:setParticleLife(1)
	p:setDirection(0)
	p:setSpread(360)
	p:setRadialAcceleration(-2000)
	p:setTangentialAcceleration(1000)
	p:start()
end

function love.draw()
	
	love.graphics.setFont( font_gridblock )
	love.graphics.setLineStyle( "rough" )
	
	-- Draw the particle system
	love.graphics.setBlendMode( "additive" )
	love.graphics.draw( p, 0, 0 )
	
	-- Draw the GUI
	love.graphics.setBlendMode( "subtractive" )
	love.graphics.setColor( 255, 255, 255, 70 )
	love.graphics.rectangle( "fill", 0, 0, 150, love.graphics.getHeight() )
	love.graphics.rectangle( "fill", 149, 0, 1, love.graphics.getHeight() )
	
	-- Playback Control	
	love.graphics.setBlendMode( "alpha" )
	love.graphics.setColor( 255, 255, 255, 255 )
	love.graphics.draw( s.play and img_play_on or img_play, 5, 5 )
	love.graphics.draw( s.play and img_pause or img_pause_on, 26, 5 )
	love.graphics.draw( img_step, 47, 5 )
	
	-- Properties
	
	love.graphics.setColor( 200, 200, 200, 255 )
	love.graphics.setLineWidth( 1 )
	
	love.graphics.print( "Buffer Size", 4, 40 )
	love.graphics.rectangle( "line", 5, 50, 139, 13 )
	love.graphics.rectangle( "fill", 6, 51, 136, 10 )
	
	-- Get Tooltip
	local x, y = love.mouse.getX(), love.mouse.getY()
	local str = ""
	if x >= 5 and x <= 21 and y >= 5 and y <= 21 then
		str = "Start the particle system"
	end
	if x >= 26 and x <= 42 and y >= 5 and y <= 21 then
		str = "Pause the particle system"
	end
	if x >= 47 and x <= 63 and y >= 5 and y <= 21 then
		str = "Advance the particle system one frame"
	end
	
	love.graphics.setColor( 0, 0, 0, 255 )
	for i = -1, 1 do
		for j = -1, 1 do
			love.graphics.print( str, 155 + i, love.graphics.getHeight() - 12 + j )
		end
	end
	love.graphics.setColor( 255, 255, 255, 255 )
	love.graphics.print( str, 155, love.graphics.getHeight() - 12 )
end

function love.update( dt )
	
	local x, y = love.mouse.getX(), love.mouse.getY()
	
	-- Move the particle system
	if love.mouse.isDown( "l" ) and x > 150 then
		p:setPosition( x, y )
	end
	
	-- Update the system
	if s.play or s.step then
		p:update( dt )
		s.step = false
	end
	
end

function love.mousepressed( x, y, button )
	
	-- Detect clicking
	if button == "l" then
		
		-- Play
		if x >= 5 and x <= 21 and y >= 5 and y <= 21 then
			s.play = true
		end
		
		-- Pause
		if x >= 26 and x <= 42 and y >= 5 and y <= 21 then
			s.play = false
		end
		
		-- Step
		if x >= 47 and x <= 63 and y >= 5 and y <= 21 then
			s.step = true
			s.play = false
		end
		
	end
	
end